
#include <string.h>
#include <libxml/xmlmemory.h>
#include <libxml/debugXML.h>
#include <libxml/HTMLtree.h>
#include <libxml/xmlIO.h>
#include <libxml/DOCBparser.h>
#include <libxml/xinclude.h>
#include <libxml/catalog.h>
#include <libxslt/xslt.h>
#include <libxslt/xsltInternals.h>
#include <libxslt/transform.h>
#include <libxslt/xsltutils.h>

#include <gtkhtml/gtkhtml.h>
#include <gnome.h>

#include "common.h"
#include "transform.h"

extern int xmlLoadExtDtdDefaultValue;


int xml_xslt_init (Data *data)
{
  xmlSubstituteEntitiesDefault(1);
  xmlLoadExtDtdDefaultValue = 1;

  data->doc_tree = NULL;
  data->xslt_tree = NULL;
  data->html_tree = NULL;
  return 0;
}

int xml_xslt_close (Data *data)
{
  if (data->xslt_tree) xsltFreeStylesheet(data->xslt_tree);
  if (data->html_tree) xmlFreeDoc(data->html_tree);
  if (data->doc_tree) xmlFreeDoc(data->doc_tree);

  xsltCleanupGlobals();
  xmlCleanupParser();
  return 0;
}


int xslt_load_stylesheet(Data *data)
{
  xsltStylesheetPtr cur = data->xslt_tree;
  const char *text_loading = "<html>loading stylesheet... ";
  const char *text_failed = "failed</html>";
  const char *text_done = "done</html>";
  
  if (cur) xsltFreeStylesheet(data->xslt_tree);

  data->stream = gtk_html_begin(GTK_HTML (data->html));
  gtk_html_write (GTK_HTML (data->html), data->stream, text_loading, strlen(text_loading));
  //gtk_html_end (GTK_HTML (data->html), data->stream, GTK_HTML_STREAM_OK);

  g_message("Loading stylesheet %s", data->xslt_name);

  cur = xsltParseStylesheetFile(data->xslt_name);
  if (cur == NULL) {
    g_warning("cannot parse stylesheet %s", data->xslt_name);
    gtk_html_write (GTK_HTML (data->html), data->stream, text_failed, strlen(text_failed));
  }
  else {
    g_message("Done");
    gtk_html_write (GTK_HTML (data->html), data->stream, text_done, strlen(text_done));
  }

  gtk_html_end (GTK_HTML (data->html), data->stream, GTK_HTML_STREAM_OK);

  data->xslt_tree = cur;
  return 0;
}

int xml_load_document(Data *data)
{
  xmlDocPtr doc = data->doc_tree;
  const char *text_loading = "<html>loading document... ";
  const char *text_failed = "failed</html>";
  const char *text_done = "done</html>";

  if (doc) xmlFreeDoc(doc);

  data->stream = gtk_html_begin(GTK_HTML (data->html));
  gtk_html_write (GTK_HTML (data->html), data->stream, text_loading, strlen(text_loading));
  //gtk_html_end (GTK_HTML (data->html), data->stream, GTK_HTML_STREAM_OK);
  g_message("Loading document %s", data->doc_name);
  
  doc = xmlParseFile(data->doc_name);
  if (doc == NULL) {
    g_warning("cannot parse document %s", data->doc_name);
    gtk_html_write (GTK_HTML (data->html), data->stream, text_failed, strlen(text_failed));
  }
  else {
    g_message("Done");
    gtk_html_write (GTK_HTML (data->html), data->stream, text_done, strlen(text_done));
  }

  gtk_html_end (GTK_HTML (data->html), data->stream, GTK_HTML_STREAM_OK);

  data->doc_tree = doc;
  return 0;
}


int xslt_transform (Data *data)
{
  xsltStylesheetPtr cur = data->xslt_tree;
  xmlDocPtr doc = data->doc_tree;
  xmlDocPtr res = data->html_tree;

  xmlChar *buf;
  int len;

  data->stream = gtk_html_begin(GTK_HTML (data->html));
  
  if (doc == NULL) {
    gtk_html_write (GTK_HTML (data->html), data->stream, "<html>no document loaded</html>", len);
    g_warning("no document");
    return 0;
  }

  if (cur == NULL) {
    gtk_html_write (GTK_HTML (data->html), data->stream, "<html>no stylesheet loaded</html>", len);
    g_warning("no stylesheet");
    return 0;
  }


  /* here is real conversion */
  g_message("start processing");
  res = xsltApplyStylesheet(cur, doc, NULL);
  g_message("processing done");
  
  if (res == NULL) {
    /* failure */
    g_warning("failed to convert");
    gtk_html_write (GTK_HTML (data->html), data->stream, "<html>failed to convert document</html>", len);
  }
  else {
    /* pass to gtkhtml */
    xmlDocDumpMemory (res, &buf, &len);
    //g_print("%s", buf);
    gtk_html_write (GTK_HTML (data->html), data->stream, buf, len);

    xmlFree(buf);
    xmlFreeDoc(res);
  }

  gtk_html_end (GTK_HTML (data->html), data->stream, GTK_HTML_STREAM_OK);
  
  return 0;
}
